#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include "nokia5110/nokia5110.h"

#define LCD_WIDTH 84
#define LCD_HEIGHT 48

#define NONE -1
#define START 0
#define UP 1
#define DOWN 2
#define LEFT 3
#define RIGHT 4

#define INITIAL_SIZE 10
#define MAX_SIZE 60
#define INITIAL_X 60
#define INITIAL_Y 24
#define BLOCK_SIZE 1

#define LOST -1

#define START_BUTTON (PINA & (1 << PA7))
#define UP_BUTTON (PINA & (1 << PA6))
#define DOWN_BUTTON (PINA & (1 << PA5))
#define LEFT_BUTTON (PINA & (1 << PA4))
#define RIGHT_BUTTON (PINA & (1 << PA3))


typedef struct {
	uint8_t x;
	uint8_t y;
} coordinates;


coordinates food;
coordinates snake[MAX_SIZE];

uint32_t game;
uint32_t head, tail, direction, size;

/* Butoanele sunt puse ca intrare, cu rezistenta de pull-up */
void init_buttons(void) {
	/* START */
    DDRA &= ~(1 << PA7);
    PORTA |= (1 << PA7);

	/* UP */
    DDRA &= ~(1 << PA6);
    PORTA |= (1 << PA6);

    /* DOWN */
    DDRA &= ~(1 << PA5);
    PORTA |= (1 << PA5);
    
    /* LEFT */
    DDRA &= ~(1 << PA4);
    PORTA |= (1 << PA4);

    /* RIGHT */
    DDRA &= ~(1 << PA3);
    PORTA |= (1 << PA3);    
}

/* Verific ce buton e apasat */
void check_button(void) {
	/* START */
    if (START_BUTTON == 0) {
        direction = START;
    }
    
    /* UP */
    if (UP_BUTTON == 0) {
        direction = UP;
    }
    
    /* DOWN */
    if (DOWN_BUTTON == 0) {
        direction = DOWN;
    }
    
    /* LEFT */
    if (LEFT_BUTTON == 0) {
        direction = LEFT;
    }

    /* RIGHT */
    if (RIGHT_BUTTON == 0) {
        direction = RIGHT;
    }
}

/* Printeaza un mesaj cu nume, grupa si proiect */
void print_initial_screen(void) {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(5, 0);
	nokia_lcd_write_string("Comarniceanu", 1);

	nokia_lcd_set_cursor(22, 12);
	nokia_lcd_write_string("George", 1);

	nokia_lcd_set_cursor(22, 24);
	nokia_lcd_write_string("334CA", 1);

	nokia_lcd_set_cursor(22, 36);
	nokia_lcd_write_string("Snake", 1);

	nokia_lcd_render();

	_delay_ms(2500);
}

/* Printeaza un mesaj de incepere a jocului */
void print_start_game_screen(void) {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(15, 18);
	nokia_lcd_write_string("Start Game", 1);

	nokia_lcd_render();
}

/* Jocul a fost castigat */
void congratulations(void) {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(15, 18);
	nokia_lcd_write_string("You won!", 1);

	nokia_lcd_render();
}

/* Jocul a fost pierdut */
void game_over(void) {
	nokia_lcd_clear();

	nokia_lcd_set_cursor(15, 18);
	nokia_lcd_write_string("You lost!", 1);

	nokia_lcd_render();
}

/* Generare coordonate mancare */
void generate_food_coordinates(void) {
	food.x = rand() % LCD_WIDTH;
	food.y = rand() % LCD_HEIGHT;
}

/* Initializare sarpe */
void initialize(void) {
	uint32_t i;
	size = INITIAL_SIZE;
	
	for (i = 0; i < size; i++) {
		snake[i].x = INITIAL_X - i * BLOCK_SIZE;
		snake[i].y = INITIAL_Y;
	}
	head =  INITIAL_SIZE - 1;
	tail = 0;
	generate_food_coordinates();

}

/* Sarpele se plimba pe harta si cand mananca hrana acesta creste in dimensiune */
void move_and_eat(void) {
	uint32_t prev;
	prev = head;
	head = (head + 1) % MAX_SIZE;
	snake[head] = snake[prev];


	if (abs(food.x - snake[head].x) < BLOCK_SIZE &&
		abs(food.y - snake[head].y) < BLOCK_SIZE) {

		size++;
		generate_food_coordinates();
	}
	else
	{
		/* Mut sfarsitul cozii */
		tail = (tail + 1) % MAX_SIZE;
	}

	switch(direction) {
		case(DOWN):
			snake[head].y = (snake[head].y + BLOCK_SIZE) % LCD_HEIGHT;
			break;
		case(UP):
			snake[head].y = (snake[head].y - BLOCK_SIZE) % LCD_HEIGHT;
			break;
		case(LEFT):
			snake[head].x = (snake[head].x - BLOCK_SIZE) % LCD_WIDTH;
			break;
		case(RIGHT):
			snake[head].x = (snake[head].x + BLOCK_SIZE) % LCD_WIDTH;
			break;
		}
}

/* Desenare sarpe si hrana */
void draw(void) {
	uint32_t i;

	for (i = 0; i < size; ++i) {
		nokia_lcd_set_pixel((snake[(tail + i) % MAX_SIZE].x - BLOCK_SIZE),
							 (snake[(tail + i) % MAX_SIZE].y - BLOCK_SIZE), 1);

	}

	nokia_lcd_set_pixel(food.x - BLOCK_SIZE, food.y - BLOCK_SIZE, 1);

	nokia_lcd_render();
}

/* Se musca singur */
void bite(void) {
	uint32_t i;
	for (i = tail; i < head; i++) {
		if (snake[i].x == snake[head].x &&
			snake[i].y == snake[head].y)

			game = LOST;
		}
}

int main(void) {
	nokia_lcd_init();
	init_buttons();
	print_initial_screen();
	print_start_game_screen();

	direction = NONE;

	while(1) {
		check_button();
		if (direction == START) {
			nokia_lcd_clear();
			break;
		}
	}

	initialize();

	while(1) {
		check_button();
		nokia_lcd_clear();
		draw();
		move_and_eat();
		_delay_ms(200);
		if (size == MAX_SIZE) {
			congratulations();
			break;
		}
	}

	return 0;

}